package shows

import (
	"fmt"

	"gitlab.quimbo.fr/odwrtw/canape-sql/sqly"
	"gitlab.quimbo.fr/odwrtw/canape-sql/users"

	"github.com/jmoiron/sqlx"
	"github.com/odwrtw/polochon/lib"
)

var Schema = sqly.Schema{
	Require: []sqly.Schema{
		users.Schema,
	},
	Tables: []sqly.SchemaTable{
		sqly.SchemaTable{
			Name: "shows",
			Sql: `
			CREATE TABLE shows (
				id SERIAL PRIMARY KEY,
				imdbid text NOT NULL UNIQUE,
				title text NOT NULL,
				updated timestamp DEFAULT current_timestamp,
				created timestamp DEFAULT current_timestamp
			);
		`},
		sqly.SchemaTable{
			Name: "episodes",
			Sql: `
			CREATE TABLE episodes (
				id SERIAL PRIMARY KEY,
				shows_id integer REFERENCES shows (id) ON DELETE CASCADE,
				title text NOT NULL,
				season integer NOT NULL,
				episode integer NOT NULL,
				updated timestamp DEFAULT current_timestamp,
				created timestamp DEFAULT current_timestamp
			);
		`},
	},
	Drop: `
		DROP TABLE episodes;
		DROP TABLE shows;
	`,
}

const (
	addShowQuery    = `INSERT INTO shows (imdbid, title) VALUES (:imdbid, :title) RETURNING id;`
	getShowQuery    = `SELECT * FROM shows WHERE imdbid=$1;`
	deleteShowQuery = `DELETE FROM shows WHERE id=$1;`

	addEpisodeQuery  = `INSERT INTO episodes (shows_id, title, season, episode) VALUES ($1,$2,$3,$4);`
	getEpisodesQuery = `SELECT title, season, episode FROM episodes WHERE shows_id=$1;`
)

type Show struct {
	sqly.BaseTable
	polochon.Show
	Episodes []*Episode
}

func Get(db *sqlx.DB, imdbID string) (*Show, error) {
	s := &Show{}
	err := db.QueryRowx(getShowQuery, imdbID).StructScan(s)
	if err != nil {
		return nil, err
	}
	return s, nil
}

func (s *Show) Add(db *sqlx.DB) error {
	var id int
	r, err := db.NamedQuery(addShowQuery, s)
	if err != nil {
		return err
	}
	for r.Next() {
		r.Scan(&id)
	}
	s.ID = id

	// When add a show to database use polochon episode details
	// so s.Show.Episodes
	for _, pEp := range s.Show.Episodes {
		err = s.addEpisode(db, pEp)
		if err != nil {
			return err
		}
	}
	return nil
}

func (s *Show) addEpisode(db *sqlx.DB, pEpisode *polochon.ShowEpisode) error {
	_, err := db.Exec(addEpisodeQuery, s.ID, pEpisode.Title, pEpisode.Season, pEpisode.Episode)
	if err != nil {
		return err
	}
	return nil
}

func (s *Show) Delete(db *sqlx.DB) error {
	r, err := db.Exec(deleteShowQuery, s.ID)
	if err != nil {
		return err
	}
	count, _ := r.RowsAffected()
	if count != 1 {
		return fmt.Errorf("Unexpected number of row deleted: %d", count)
	}
	return nil
}

func (s *Show) GetEpisodes(db *sqlx.DB) error {
	// When retrive episode's info from database populate the s.Episodes member
	// and not s.Show.Episodes
	err := db.Select(&s.Episodes, getEpisodesQuery, s.ID)
	if err != nil {
		return err
	}

	return nil
}

type Episode struct {
	sqly.BaseTable
	polochon.ShowEpisode
}
